/*
 *	Copyright (C) 1996 Be Specific Inc.
 *	All rights reserved.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 */
 


#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <AudioSubscriber.h>

#include "audio.h"
#include "ulaw.h"




//=======================================================================
//=======================================================================
bool
AudioPlaybackFunction(void *userData, char *buffer, long count)
{
	AAudioOutputDevice *aDevice = (AAudioOutputDevice*)userData;
	
	#ifdef DEBUG
	printf("AudioPlaybackFunction - Received Buffer of size: %d\n",count);
	#endif
		
	return aDevice->PlayNextPart(buffer,count);
}

//=======================================================================
//=======================================================================
AAudioOutputDevice::AAudioOutputDevice()
	: fSubscriber(0),
	fData(0),
	fSoundFile(0)
{
	fSubscriber = new BAudioSubscriber("audiodevice");

	fSubscriberID = fSubscriber->Subscribe(B_DAC_STREAM, B_INVISIBLE_SUBSCRIBER_ID, 1);
	
	fSubscriber->SetSamplingRate(8000);
	fSubscriber->SetDACSampleInfo(2,1,B_BIG_ENDIAN,B_LINEAR_SAMPLES);
}

void 
AAudioOutputDevice::Play (char * data, long datalen)
{
	fData = data;
	fDataLength = datalen;
	fOffset = 0;
	fAmountLeft = fDataLength;
	
	fSubscriber->EnterStream(0,TRUE,this,AudioPlaybackFunction,0,FALSE);
}

void 
AAudioOutputDevice::Play (BSoundFile *aFile)
{
	fData = 0;
	fDataLength = 0;
	fOffset = 0;
	fAmountLeft = fDataLength;
	fSoundFile = aFile;
	
	SetSamplingRate(aFile->SamplingRate());
	SetParameters(aFile->SampleSize(),
			aFile->FrameSize(),
			aFile->ByteOrder(),
			B_LINEAR_SAMPLES);
	
	fSubscriber->EnterStream(0,TRUE,this,AudioPlaybackFunction,0,FALSE);
}



bool
AAudioOutputDevice::PlayNextPart(char *buffer, long count)
{
	if (fData)
	{
		long smallest = 0;
		short	*aShortPtr = (short *)buffer;
	
		if ((fDataLength) < count)
			smallest = fDataLength;
		else
			smallest = count;
			
		if (smallest > 0)
		{
			for (int counter=0; counter < smallest; counter++)
				buffer[counter] += fData[fOffset+counter];
		}
	
		fOffset += smallest;
		fAmountLeft -= smallest;

		if (fAmountLeft < 1)
			return FALSE;
		else 
			return TRUE;
	} else  // Assume we're dealing with a file
	{
		if (fSoundFile->SampleSize() == 1)
		{	
			char aBuffer[4096];
			short	*aShortPtr = (short *)buffer;
			long bytesRead = fSoundFile->ReadFrames(aBuffer, count/2);	
	
			// turn the uLaw read into two byte linear
			for (int counter = 0; counter < bytesRead; counter++)
			{
				if (fSoundFile->SampleFormat() == B_MULAW_SAMPLES)
					aShortPtr[counter] += ulaw2linear(aBuffer[counter]);
				else
					aShortPtr[counter] += aBuffer[counter];	
			}
		} else
		{
			short aBuffer[4096];
			short	*aShortPtr = (short *)buffer;
			long maxFrames = count / (fSoundFile->FrameSize()*fSoundFile->SampleSize());
			long framesRead = fSoundFile->ReadFrames((char *)aBuffer, maxFrames);	
	
			//printf("Frames To Read: %d  Frames Read: %d\n",maxFrames, framesRead);	
			for (int counter = 0; counter < framesRead; counter++)
			{
				aShortPtr[counter] += aBuffer[counter];
			}
		}
		
		//printf("Frames Remaining: %d\n",fSoundFile->FramesRemaining());	
		if (fSoundFile->FramesRemaining() > 0)
		{
			return TRUE;		// We have more to play
		} else
		{
			return FALSE;	// We are out of sound files
		}
	}
}

long 
AAudioOutputDevice::SetVolume (const float l_volume, const float r_volume)
{
	return fSubscriber->SetVolume(B_DAC_OUT, l_volume, r_volume);
}

long	
AAudioOutputDevice::SetParameters(long bytesPerSample,
								long samplesPerFrame,
								long byteOrder,
								long sampleFormat)
{
	printf("AAudioOuputDevice: \n");
	printf("  BytesPerSample: %d\n",bytesPerSample);
	printf("  SamplesPerFrame: %d\n",samplesPerFrame);
	printf("  ByteOrder: %d\n",byteOrder);
	printf("  SampleFormat: %d\n",sampleFormat);
	
	return fSubscriber->SetDACSampleInfo(bytesPerSample, samplesPerFrame, byteOrder, sampleFormat);
}

long
AAudioOutputDevice::SetSamplingRate(long sRate)
{
	return fSubscriber->SetSamplingRate(sRate);
}

		








//=======================================================================
//=======================================================================


bool
muLawPlaybackFunction(void *userData, char *buffer, long count)
{
	AmuLawAudioDevice *aDevice = (AmuLawAudioDevice*)userData;
	
	#ifdef DEBUG
	printf("Received Buffer of size: %d\n",count);
	#endif
		
	return aDevice->PlayNextPart(buffer,count);
}

//=======================================================================
//=======================================================================

AmuLawAudioDevice::AmuLawAudioDevice()
	: fSubscriber(0)
{
	fSubscriber = new BAudioSubscriber("recite");

	fSubscriberID = fSubscriber->Subscribe(B_DAC_STREAM, B_INVISIBLE_SUBSCRIBER_ID, 1);
	fSubscriber->SetSamplingRate(8000);
	fSubscriber->SetDACSampleInfo(2,1,B_BIG_ENDIAN,B_LINEAR_SAMPLES);
}

void 
AmuLawAudioDevice::Play (int argc, char **argv)
{
	fArgc = argc;
	fArgv = argv;
	fCurrentSound=1;
	fMoreToPlay = TRUE;
	char aPath[1024];
	
	//getcwd(aPath,1023);
	sprintf(aPath,"%s", argv[fCurrentSound]);
	//strcat(aPath,"/");
	//strcat(aPath,argv[fCurrentSound]);
	printf("Play: %s\n", aPath);
	get_ref_for_path(aPath, &fSoundRef);
	fSoundFile.SetRef(fSoundRef);
	fSoundFile.SetFileFormat(B_UNIX_FILE);
	fSoundFile.Open(B_READ_ONLY);
	
	if (fSoundFile.Error() == B_ERROR)
	{
		printf("Error in sound file\n");
		return ;
	}
		
	while (fMoreToPlay)
	{
		fSubscriber->EnterStream(0,TRUE,this,muLawPlaybackFunction,0,FALSE);
	}
}

bool
AmuLawAudioDevice::PlayNextPart(char *buffer, long count)
{
	char aBuffer[8192];
	short	*aShortPtr = (short *)buffer;
	
	// use argc, argv to get a BSoundfile open for data
	long bytesRead = fSoundFile.ReadFrames( aBuffer, count/2);	
	#ifdef DEBUG
	printf("AmuLawAudioDevice::PlayNextPart - Bytes read: %d\n",bytesRead);
	#endif
	
	// turn the uLaw read into two byte linear
	for (int counter = 0; counter < bytesRead; counter++)
	{
		aShortPtr[counter] += ulaw2linear(aBuffer[counter]);
		
		// You could use the following, and it would be faster
		// by saving the function call and calculations, but the
		// quality isn't as good.  Try it out.
		//aShortPtr[counter] += _u2l[aBuffer[counter]];
	
	}
	
	if (fSoundFile.FramesRemaining() > 0)
	{
		return TRUE;		// We have more to play
	} else
	{
		fCurrentSound++;
		if (fCurrentSound<fArgc)
		{
			get_ref_for_path(fArgv[fCurrentSound], &fSoundRef);
			fSoundFile.SetRef(fSoundRef);
			fSoundFile.Open(B_READ_ONLY);
			return TRUE;
		} else
		{
			fMoreToPlay = FALSE;
			return FALSE;	// We are out of sound files
		}
	}
}

void 
AmuLawAudioDevice::SetVolume (const double volume)
{
	// fSubscriber->SetVolumn(MASTER_DEVICE, volume, volume);
}

void 
AmuLawAudioDevice::Read (char *filename, char **data, long *datalen)
{
}

void 
AmuLawAudioDevice::Write (char *filename, char *data, long datalen)
{
}


