/*
** COPYRIGHT
**
** This code is so simple, it's not even worth a copyright
** notice.  It's public domain, do what you want with it.
**
** Author: William Adams
*/

#include <Application.h>
#include <Beep.h>
#include <Store.h>
#include <SoundFile.h>
#include "audio.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


int optind=1;
char *optarg;
int opterr=1;

int getopt (int argc, char **argv, char *pat)
{
  char *flag;
  
  if((optind >= argc) || (argv[optind][0] != '-'))
  {
    return -1;
  }

  if( argv[optind][1] == '-' )
  {
    optind++;
    return -1;
  }

  if( argv[optind][1] == ':' )
  {
    if( opterr )
    {
      printf("getopt: found \":\" in optstring\n");
    }
    return '?';
  }

  for(flag=pat;*flag;flag++)
  {
    if( *flag == argv[optind][1] )
    {
      optind++;
      if( *(flag+1) == ':' )
      {
        if(optind >= argc )
        {
          if( opterr )
          {
            printf("getopt: no option for flag %c\n",*flag);
          }
          return '?';
        }
        optarg = argv[optind];
        optind++;
      }
      return *flag;
    }
      
  }
  if( opterr )
  {
    printf("getopt: flag %s not recognized\n",argv[optind]);
  }
  optind++;
  return '?';
}

//===========================================================
// Class: SoundApplication
//
// This is a simple application class to get things started
//===========================================================


class SoundApplication : public BApplication
{
public:
					SoundApplication();
		
	virtual	void	ArgvReceived(int argc, char **argv);
	virtual	void	ReadyToRun();
	
protected:
	record_ref	fSoundRef;
	char fFileName[1024];
	
	long fsampleFormat;
	long fchannelCount;
	long fsamplingRate;
	long fbyteOrder;
	long fbytesPerSample;

private:

};


//===========================================================
// Class: SoundApplication
//
// This is the interface to the sound application
//===========================================================

SoundApplication::SoundApplication()
	: BApplication('ASND')
{
	fsampleFormat = B_MULAW_SAMPLES;
	fchannelCount = 1;
	fsamplingRate = 8000;
	fbyteOrder = B_BIG_ENDIAN;
	fbytesPerSample = 2;
}

void
SoundApplication::ArgvReceived(int argc, char **argv)
{	
	if (argc < 2)
		puts("Usage: playsound <soundfile>");
		
	printf("ArgvReceived\n");
	
	int c;
	
	while((c = getopt(argc,argv,"b:B:f:r:c:"))!= -1)
	{
		printf("OptArg: %s\n",optarg);
		switch(c)
		{
			case 'b':	// Bytes per sample
				fbytesPerSample = atoi(optarg);
				printf("Bytes Per Sample opt: %d\n",fbytesPerSample);
			break;
				
			case 'B':	// Byte ordering
				//bigendian ==0
				//littleendian == 1
				fbyteOrder = atol(optarg);
			break;
			
			case 'f':	// sample Format
				// UNDEFINED == 0
				// LINEAR == 1
				// FLOAT == 2
				// MULAW == 3
				fsampleFormat = atol(optarg);
			break;
			
			case 'r':	// Sampling Rate
				fsamplingRate = atol(optarg);
			break;
	
			case 'c':	// Samples Per Frame
				fchannelCount = atol(optarg);
			break;
		}
	}
	
	printf("File Name: %s\n",argv[optind]);
	strcpy(fFileName, argv[optind]);	


	// Get a reference for the sound file, and hold it.
	long error = get_ref_for_path(fFileName, &fSoundRef);
	if ( error == B_ERROR)
	{
		printf(" get_ref_for_path returned error\n");
		return ;
	}	
	printf(" get_ref_for_path returned: %d\n",error);

	if (!does_ref_conform(fSoundRef, "File"))
	{
		printf("Record ref does not conform!\n");
		return ;
	}
	
	//		printf("Playing UNIX sound file\n");
	//		wait_for_sound(play_sound(fSoundRef, 1, 1, 1));

	//return ;
	
	BSoundFile soundFile(fSoundRef);
	soundFile.SetRef(fSoundRef);
	soundFile.Open(B_READ_ONLY);
	
	switch (soundFile.FileFormat())
	{
		case B_UNIX_FILE:
			printf("Playing UNIX sound file\n");
			wait_for_sound(play_sound(fSoundRef, 1, 1, 1));
		break;
		
		case B_WAVE_FILE:
			printf("Playing WAV sound file\n");
			wait_for_sound(play_sound(fSoundRef, 1, 1, 1));
		break;
		
		case B_AIFF_FILE:
			printf("Playing AIFF sound file\n");
			wait_for_sound(play_sound(fSoundRef, 1, 1, 1));
		break;
				
		case B_UNKNOWN_FILE:
		{
			printf("Playing UNKNOWN sound file\n");
			soundFile.SetSampleFormat(fsampleFormat);
			soundFile.SetChannelCount(fchannelCount);
			soundFile.SetSampleSize(fbytesPerSample);
			soundFile.SetByteOrder(fbyteOrder);
			soundFile.SetSamplingRate(fsamplingRate);
			AAudioOutputDevice outputDevice;
			outputDevice.Play(&soundFile);
		}
		break;
		
		default:
			printf("Unknown file format: %d\n", soundFile.FileFormat());
		break;
				
	}
}

void
SoundApplication::ReadyToRun()
{
	printf("ReadyToRun\n");
	
/*
	// Get a reference for the sound file, and hold it.
	get_ref_for_path(fFileName, &fSoundRef);	

	
	BSoundFile soundFile;
	soundFile.SetRef(fSoundRef);
	
	switch (soundFile.FileFormat())
	{
		case B_UNIX_FILE:
			printf("Playing UNIX sound file\n");
			wait_for_sound(play_sound(fSoundRef, 1, 1, 1));
		break;
		
		case B_WAVE_FILE:
			printf("Playing WAV sound file\n");
			wait_for_sound(play_sound(fSoundRef, 1, 1, 1));
		break;
		
		case B_AIFF_FILE:
			printf("Playing AIFF sound file\n");
			wait_for_sound(play_sound(fSoundRef, 1, 1, 1));
		break;
				
		case B_UNKNOWN_FILE:
		{
			printf("Playing UNKNOWN sound file\n");
			soundFile.SetSampleFormat(fsampleFormat);
			soundFile.SetChannelCount(fchannelCount);
			soundFile.SetSampleSize(fbytesPerSample);
			soundFile.SetByteOrder(fbyteOrder);
			soundFile.SetSamplingRate(fsamplingRate);
			AAudioOutputDevice outputDevice;
			outputDevice.Play(&soundFile);
		}
		break;
		
		default:
			printf("Unknown file format: %d\n", soundFile.FileFormat());
		break;
				
	}
*/

	PostMessage(B_QUIT_REQUESTED);
}

int
main()
{
	SoundApplication *my_app = new SoundApplication();
	
	my_app->Run();
	delete my_app;
}